import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.IOException;

public class SaxExample extends DefaultHandler
{

	//=====================================================================
	// Main Method:
	//=====================================================================

	public static void main(String args[]) {

		// Create XML Reader:
		XMLReader xr = null;
		try {
			//String parserName =
			//	"org.apache.xerces.parsers.SAXParser";
			//xr = XMLReaderFactory.createXMLReader(parserName);
			xr = XMLReaderFactory.createXMLReader();
			SaxExample handler = new SaxExample();
			xr.setContentHandler(handler);
			xr.setErrorHandler(handler);
		} catch(Exception e) {
			System.err.println("Exception creating XMLReader: " +
						e);
			e.printStackTrace(System.err);
			System.exit(1);
		}

		// Open Input File:
		String input_file = "test.xml";
		if(args.length > 0)
			input_file = args[0];
		InputStreamReader in = null;
		try {
			FileInputStream is =
				new FileInputStream(input_file);
			in = new InputStreamReader(is, "ISO-8859-1");
		} catch(IOException e) {
			System.err.println("Error opening input file "+
				"'" + input_file + "': " + e);
			System.exit(2);
		}

		// Parse the XML input file:
		try {
			xr.parse(new InputSource(in));
			//xr.parse(new InputSource(in), new SaxExample());
		} catch(Exception e) {
			System.err.println("Error parsing " +
				"'" + input_file + "': " + e);
			System.exit(3);
		}
	}


	//=====================================================================
	// Constructor:
	//=====================================================================

	// Not necessary, this is the default version:
	public SaxExample() {
		super();
	}

	//=====================================================================
	// Event Handlers:
	//=====================================================================


	public void startDocument() {
		System.out.println("Start document");
	}


	public void endDocument() {
		System.out.println("End document");
	}


	public void startElement(String uri, String name,
			      String qName, Attributes attrs) {

		//if(name == null || name.equals(""))
		//	name = qName;
		if ("".equals(uri))
			System.out.println("Start element: " + qName);
		else
			System.out.println("Start element: {" + uri + "}" +
						name);
		int attrCount = attrs.getLength();
		if(attrCount>0) {
			System.out.println("Attributes:"); 
			for(int i = 0 ; i < attrCount; i++) {
				System.out.println("  Name : " +
							attrs.getQName(i)); 
				System.out.println("  Type : " +
							attrs.getType(i)); 
				System.out.println("  Value: \"" +
							attrs.getValue(i)+"\"");
				System.out.println();
			}
		}
	}


	public void endElement(String uri, String name, String qName) {
		//if ("".equals (uri))
		//	System.out.println("End element: " + qName);
		//else
		//	System.out.println("End element:   {" + uri + "}" +
		//				name);

		if(name == null || name.equals(""))
			name = qName;
		System.out.println("End Element: " + name);
	}


	public void characters(char ch[], int start, int length) {
		System.out.print("Text:    \"");
		for (int i = start; i < start + length; i++) {
			System.out.print(ch[i]);
		}
		System.out.println("\"");
	}

	public void warning(SAXParseException spe) {
		System.err.println("Warning at line "+spe.getLineNumber());
		System.err.println(spe.getMessage());
	}

	public void fatalError(SAXParseException spe) throws SAXException {
		System.err.println("Fatal error at line "+spe.getLineNumber());
		System.err.println(spe.getMessage());
		throw spe;
	}

}
